@@ -3,10 +3,11 @@ package Mail::MtPolicyd::Plugin::SqlUserConfig;
 use Moose;
 use namespace::autoclean;
 
-our $VERSION = '1.14'; # VERSION
+our $VERSION = '1.15'; # VERSION
 # ABSTRACT: mtpolicyd plugin for retrieving the user config of a user
 
 extends 'Mail::MtPolicyd::Plugin';
+with 'Mail::MtPolicyd::Plugin::Role::SqlUtils';
 
 
 use Mail::MtPolicyd::Plugin::Result;
@@ -28,9 +29,7 @@ sub _get_config {
 	my ( $self, $r ) = @_;
 	my $rcpt = $r->attr('recipient');
 
-	my $dbh = $r->server->get_dbh;
-	my $sth = $dbh->prepare( $self->sql_query );
-	$sth->execute( $rcpt );
+	my $sth = $self->execute_sql( $self->sql_query, $rcpt );
 	my ( $json ) = $sth->fetchrow_array;
 	if( ! defined $json ) {
 		die( 'no user-config found for '.$rcpt );
@@ -65,17 +64,19 @@ __PACKAGE__->meta->make_immutable;
 
 1;
 
-
 __END__
+
 =pod
 
+=encoding UTF-8
+
 =head1 NAME
 
 Mail::MtPolicyd::Plugin::SqlUserConfig - mtpolicyd plugin for retrieving the user config of a user
 
 =head1 VERSION
 
-version 1.14
+version 1.15
 
 =head1 DESCRIPTION
 
@@ -141,4 +142,3 @@ This is free software, licensed under:
   The GNU General Public License, Version 2, June 1991
 
 =cut
-
